import * as tslib_1 from "tslib";
import { toRect, brushToString, stringToBrush, ensureBool, initializePropertiesFromCss, NamePatcher, toSpinal, getAllPropertyNames, fromSpinal } from "./componentUtil";
import { WebComponentRenderer, PortalManager } from "./web-component-renderer";
import { delegateCombine, delegateRemove } from "./type";
import { RegisterElementHelper } from "./module-manager";
import { Popup } from "./Popup";
import { IgcOnPopupEventArgs } from './igc-on-popup-event-args';
import { IgcOnClosedEventArgs } from './igc-on-closed-event-args';
import { IgcHTMLElement } from "./igc-html-element";
var IgcPopupComponent = /** @class */ /*@__PURE__*/ (function (_super) {
    tslib_1.__extends(IgcPopupComponent, _super);
    function IgcPopupComponent() {
        var _this = _super.call(this) || this;
        _this._implementation = null;
        _this._disconnected = false;
        _this._settingAttributes = false;
        _this._attached = false;
        _this._queuedSetAttributes = [];
        _this._updatingFromAttribute = false;
        _this.__p = null;
        _this._hasUserValues = new Set();
        _this._stylingContainer = null;
        _this._stylingParent = null;
        _this._inStyling = false;
        _this._onClosed = null;
        _this._onClosed_wrapped = null;
        _this._onPopup = null;
        _this._onPopup_wrapped = null;
        if (_this._styling) {
            NamePatcher.ensureStylablePatched(Object.getPrototypeOf(_this));
        }
        _this._implementation = _this.createImplementation();
        _this._portalManager = new PortalManager("popupContent");
        _this._webComponentRenderer = new WebComponentRenderer(_this, document, true, {}, _this._portalManager);
        _this._mainDiv = _this._webComponentRenderer.createElement("div");
        _this._webComponentRenderer.updateRoot(_this._mainDiv);
        _this.i.provideRenderer(_this._webComponentRenderer);
        var mut = new MutationObserver(function (list) {
            var e_1, _b;
            try {
                for (var list_1 = tslib_1.__values(list), list_1_1 = list_1.next(); !list_1_1.done; list_1_1 = list_1.next()) {
                    var mutation = list_1_1.value;
                    if (mutation.type == 'childList') {
                        _this._onChildrenChanged();
                    }
                }
            }
            catch (e_1_1) {
                e_1 = { error: e_1_1 };
            }
            finally {
                try {
                    if (list_1_1 && !list_1_1.done && (_b = list_1.return))
                        _b.call(list_1);
                }
                finally {
                    if (e_1)
                        throw e_1.error;
                }
            }
        });
        mut.observe(_this, {
            childList: true
        });
        return _this;
    }
    Object.defineProperty(IgcPopupComponent.prototype, "height", {
        get: function () {
            return this._height;
        },
        set: function (value) {
            this._height = value;
            this.style.height = value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "width", {
        get: function () {
            return this._width;
        },
        set: function (value) {
            this._width = value;
            this.style.width = value;
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent.prototype._onChildrenChanged = function () {
        var children = [];
        for (var i = 0; i < this.childNodes.length; i++) {
            var child = this.childNodes[i];
            if (child !== this._mainDiv.getNativeElement()) {
                children.push(child);
            }
        }
        this._portalManager.onChildrenChanged(children);
    };
    // supports angular themes or custom properties set in CSS
    IgcPopupComponent.prototype.updateStyle = function () {
        this._styling(this, this);
    };
    IgcPopupComponent.prototype.destroy = function () {
        this._implementation.destroy();
        this._webComponentRenderer.destroy();
    };
    IgcPopupComponent.prototype.createImplementation = function () {
        return new Popup();
    };
    Object.defineProperty(IgcPopupComponent.prototype, "i", {
        get: function () {
            return this._implementation;
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent.prototype.disconnectedCallback = function () {
        this._disconnected = true;
    };
    IgcPopupComponent.prototype.connectedCallback = function () {
        if (this._disconnected) {
            this._disconnected = false;
            return;
        }
        this.classList.add("ig-popup");
        this.classList.add("igr-popup");
        this.appendChild(this._mainDiv.getNativeElement());
        this._attached = true;
        this.style.display = "block";
        this.style.height = this._height;
        this.style.width = this._width;
        this._flushQueuedAttributes();
        // supports themes or custom properties set in CSS
        this._styling(this, this);
        this.afterContentInit();
    };
    IgcPopupComponent.prototype.afterContentInit = function () {
    };
    IgcPopupComponent.prototype._enqueueSetAttribute = function (attrName, attrValue) {
        this._queuedSetAttributes.push({ attrName: attrName, attrValue: attrValue });
    };
    IgcPopupComponent.prototype._flushQueuedAttributes = function () {
        this._settingAttributes = true;
        for (var i = 0; i < this._queuedSetAttributes.length; i++) {
            this.setAttribute(this._queuedSetAttributes[i].attrName, this._queuedSetAttributes[i].attrValue);
        }
        this._settingAttributes = false;
        this._queuedSetAttributes.length = 0;
    };
    IgcPopupComponent.prototype._a = function (attrName, attrValue) {
        if (this._updatingFromAttribute) {
            return;
        }
        if (attrValue) {
            attrValue = attrValue.toString();
        }
        this._settingAttributes = true;
        attrName = toSpinal(attrName);
        if (this._attached) {
            this.setAttribute(attrName, attrValue);
        }
        else {
            this._enqueueSetAttribute(attrName, attrValue);
        }
        this._settingAttributes = false;
    };
    Object.defineProperty(IgcPopupComponent, "observedAttributes", {
        get: function () {
            if (IgcPopupComponent._observedAttributesIgcPopupComponent == null) {
                var names = getAllPropertyNames(IgcPopupComponent);
                for (var i = 0; i < names.length; i++) {
                    names[i] = toSpinal(names[i]);
                }
                IgcPopupComponent._observedAttributesIgcPopupComponent = names;
            }
            return IgcPopupComponent._observedAttributesIgcPopupComponent;
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent.prototype.attributeChangedCallback = function (name, oldValue, newValue) {
        if (this._settingAttributes) {
            return;
        }
        var setName = fromSpinal(name);
        this._updatingFromAttribute = true;
        this[setName] = newValue;
        this._updatingFromAttribute = false;
    };
    IgcPopupComponent.register = function () {
        if (!IgcPopupComponent._isElementRegistered) {
            IgcPopupComponent._isElementRegistered = true;
            RegisterElementHelper.registerElement(IgcPopupComponent.htmlTagName, IgcPopupComponent);
        }
    };
    Object.defineProperty(IgcPopupComponent.prototype, "animationDuration", {
        /**
     * Gets or sets the duration of the Popup and Closeup animations in milliseconds.
    */
        get: function () {
            return this.i.y;
        },
        set: function (v) {
            this.i.y = +v;
            this._a("animationDuration", this.i.y);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "animationEnabled", {
        /**
         * Gets or sets whether the popup and closeup animations are animated.
        */
        get: function () {
            return this.i.q;
        },
        set: function (v) {
            this.i.q = ensureBool(v);
            this._a("animationEnabled", this.i.q);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "elevation", {
        /**
         * Gets or sets the elevation to use for the button regardless of type.
        */
        get: function () {
            return this.i.aa;
        },
        set: function (v) {
            this.i.aa = +v;
            this._a("elevation", this.i.aa);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualElevation", {
        /**
         * Gets or sets the elevation of the border shadow
        */
        get: function () {
            return this.i.w;
        },
        set: function (v) {
            this.i.w = +v;
            this._a("actualElevation", this.i.w);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "isShown", {
        /**
         * Gets / sets the isShown state of the panel.
        */
        get: function () {
            return this.i.t;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualUmbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.a7);
        },
        set: function (v) {
            this.i.a7 = stringToBrush(v);
            this._a("actualUmbraShadowColor", brushToString(this.i.a7));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualPenumbraShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.a6);
        },
        set: function (v) {
            this.i.a6 = stringToBrush(v);
            this._a("actualPenumbraShadowColor", brushToString(this.i.a6));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "actualAmbientShadowColor", {
        /**
         * Gets or sets the color to use for the check mark when the checkbox is checked.
        */
        get: function () {
            return brushToString(this.i.a5);
        },
        set: function (v) {
            this.i.a5 = stringToBrush(v);
            this._a("actualAmbientShadowColor", brushToString(this.i.a5));
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "background", {
        /**
         * Gets or sets the background color of the popup.
        */
        get: function () {
            return brushToString(this.i.a8);
        },
        set: function (v) {
            this.i.a8 = stringToBrush(v);
            this._a("background", brushToString(this.i.a8));
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent.prototype.findByName = function (name) {
        if (this.findEphemera) {
            if (name && name.indexOf("@@e:") == 0) {
                return this.findEphemera(name);
            }
        }
        return null;
    };
    Object.defineProperty(IgcPopupComponent.prototype, "hasUserValues", {
        get: function () {
            return this._hasUserValues;
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent.prototype.__m = function (propertyName) {
        if (!this._inStyling) {
            this._hasUserValues.add(propertyName);
        }
    };
    IgcPopupComponent.prototype._styling = function (container, component, parent) {
        if (this._inStyling) {
            return;
        }
        this._inStyling = true;
        this._stylingContainer = container;
        this._stylingParent = component;
        var genericPrefix = "";
        var typeName = this.i.$type.name;
        if (typeName.indexOf("Xam") === 0) {
            typeName = typeName.substring(3);
        }
        genericPrefix = toSpinal("PopupComponent");
        var additionalPrefixes = [];
        var prefix = toSpinal(typeName);
        additionalPrefixes.push(prefix + "-");
        var b = this.i.$type.baseType;
        while (b && b.name != "Object" &&
            b.name != "Base" &&
            b.name != "Control" &&
            b.Name != "DependencyObject" &&
            b.Name != "FrameworkElement") {
            typeName = b.name;
            if (typeName.indexOf("Xam") === 0) {
                typeName = typeName.substring(3);
            }
            var basePrefix = toSpinal(typeName);
            additionalPrefixes.push(basePrefix + "-");
            b = b.baseType;
        }
        if (parent) {
            var parentTypeName = parent.i.$type.name;
            if (parentTypeName.indexOf("Xam") === 0) {
                parentTypeName = parentTypeName.substring(3);
            }
            var parentPrefix = toSpinal(parentTypeName);
            additionalPrefixes.push(parentPrefix + "-" + genericPrefix + "-");
            additionalPrefixes.push(parentPrefix + "-" + prefix + "-");
        }
        initializePropertiesFromCss(container, this, genericPrefix + "-", this.hasUserValues, false, additionalPrefixes);
        if (this._otherStyling) {
            this._otherStyling(container, component, parent);
        }
        this._inStyling = false;
    };
    /**
 * Closes the popup.

*/
    IgcPopupComponent.prototype.close = function () {
        this.i.ao();
    };
    IgcPopupComponent.prototype.showRelativeToExclusionRect = function (exclusionRect, popupDirection, popupAlignment) {
        this.i.showRelativeToExclusionRect(toRect(exclusionRect), (popupDirection == null ? null : popupDirection), (popupAlignment == null ? null : popupAlignment));
    };
    /**
     * Exports visual information about the current state of the grid.
    
    */
    IgcPopupComponent.prototype.exportVisualModel = function () {
        var iv = this.i.ab();
        return (iv);
    };
    /**
     * Returns a serialized copy of the exported visual model
    
    */
    IgcPopupComponent.prototype.exportSerializedVisualModel = function () {
        var iv = this.i.ak();
        return (iv);
    };
    Object.defineProperty(IgcPopupComponent.prototype, "onClosed", {
        get: function () {
            return this._onClosed;
        },
        set: function (ev) {
            var _this = this;
            if (this._onClosed_wrapped !== null) {
                this.i.onClosed = delegateRemove(this.i.onClosed, this._onClosed_wrapped);
                this._onClosed_wrapped = null;
                this._onClosed = null;
            }
            this._onClosed = ev;
            this._onClosed_wrapped = function (o, e) {
                var outerArgs = new IgcOnClosedEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnClosed) {
                    _this.beforeOnClosed(_this, outerArgs);
                }
                if (_this._onClosed) {
                    _this._onClosed(_this, outerArgs);
                }
            };
            this.i.onClosed = delegateCombine(this.i.onClosed, this._onClosed_wrapped);
            ;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgcPopupComponent.prototype, "onPopup", {
        /**
         * Fires when the popup is shown
        */
        get: function () {
            return this._onPopup;
        },
        set: function (ev) {
            var _this = this;
            if (this._onPopup_wrapped !== null) {
                this.i.onPopup = delegateRemove(this.i.onPopup, this._onPopup_wrapped);
                this._onPopup_wrapped = null;
                this._onPopup = null;
            }
            this._onPopup = ev;
            this._onPopup_wrapped = function (o, e) {
                var outerArgs = new IgcOnPopupEventArgs();
                outerArgs._provideImplementation(e);
                if (_this.beforeOnPopup) {
                    _this.beforeOnPopup(_this, outerArgs);
                }
                if (_this._onPopup) {
                    _this._onPopup(_this, outerArgs);
                }
            };
            this.i.onPopup = delegateCombine(this.i.onPopup, this._onPopup_wrapped);
            ;
        },
        enumerable: true,
        configurable: true
    });
    IgcPopupComponent._observedAttributesIgcPopupComponent = null;
    IgcPopupComponent.htmlTagName = "igc-popup";
    IgcPopupComponent._isElementRegistered = false;
    return IgcPopupComponent;
}(IgcHTMLElement));
export { IgcPopupComponent };
